import click
from datetime import datetime
import os
from pathlib import Path

"""
    Copyright 2019 Shabd Saran

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""


def create_license() -> None:
    """utility function to create license file"""

    # prompt to check if the user wants to create a license file
    is_license: bool = click.prompt(
        'do you want to create license file?', default=False, type=bool)
    if is_license is True:
        # prompt the user for license owner type
        license_type: str = click.prompt(
            'license type?', type=click.Choice(['MIT', 'Apache 2.0'], case_sensitive=False))
        license_type = license_type.strip().upper()

        # prompt the user for license owner name
        license_owner: str = click.prompt(
            'license owner name?', default="<license-owner>", type=str)
        license_owner = license_owner.strip().upper()

        # prompt the user for license year (default set to current year)
        license_year: int = click.prompt(
            'license year?', default=datetime.now().year, type=int)

        # load license
        if 'MIT' == license_type:
            from data.licenses.MIT import LICENSE
        else:  # apache 2.0
            from data.licenses.APACHE import LICENSE

        # fill license with user details
        LICENSE = LICENSE.replace('[year]', str(license_year))
        LICENSE = LICENSE.replace('[fullname]', license_owner)

        # create license file
        with open('LICENSE', 'w') as file:
            file.write(LICENSE)

        # load license notice
        if 'MIT' == license_type:
            from data.license_notices.MIT import LICENSE as LICENSE_NOTICE
        else:  # apache 2.0
            from data.license_notices.APACHE import LICENSE as LICENSE_NOTICE

        # fill license with user details
        LICENSE_NOTICE = LICENSE_NOTICE.replace('[year]', str(license_year))
        LICENSE_NOTICE = LICENSE_NOTICE.replace('[fullname]', license_owner)

        # create license notice file
        with open('LICENSE_NOTICE', 'w') as file:
            file.write(LICENSE_NOTICE)


def create_readme() -> None:
    """utility function to create README.md"""

    # get project directory name
    dir_name: str = str(os.getcwd()).split('/')[-1]

    # load content
    from data.README import README
    README = README.replace('[title]', dir_name)

    # create file
    with open('README.md', 'w') as file:
        file.write(README)


def create_gitignore(language: str) -> None:
    """utility function to create .gitignore"""

    # load content
    if 'c' == language or 'cpp' == language:
        from data.gitignore.cpp import GITIGNORE
    elif 'java' == language:
        from data.gitignore.java import GITIGNORE
    elif 'py' == language:
        from data.gitignore.py import GITIGNORE
    else:  # js
        from data.gitignore.js import GITIGNORE

    # create file
    with open('.gitignore', 'w') as file:
        file.write(GITIGNORE)


def check_license_notice() -> bool:
    """utility function to check if current directory contains license notice"""

    return os.path.isfile('LICENSE_NOTICE')


def create_empty_file(path: str) -> None:
    """utility function to create a new empty file"""

    Path(path).touch()


def create_file_with_license_notice(path: str, file_extension: str) -> None:
    """utility function to create a new file with license notice included"""

    # load license notice
    with open('LICENSE_NOTICE', 'r') as file:
        license_notice: str = file.read()

    # load new file template
    if 'c' == file_extension or 'cpp' == file_extension:
        from data.new_file.cpp import FILE_CONTENT
    elif 'java' == file_extension:
        from data.new_file.java import FILE_CONTENT
    elif 'js' == file_extension:
        from data.new_file.js import FILE_CONTENT
    else:  # py
        from data.new_file.py import FILE_CONTENT

    # load user license notice into new file template
    FILE_CONTENT = FILE_CONTENT.replace('[NOTICE]', license_notice)

    # create new file using the template
    with open(path, 'w') as file:
        file.write(FILE_CONTENT)



import click

# import utils
import utils.file as file_utils
import utils.language as lang_utils

"""
    Copyright 2019 Shabd Saran

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""


@click.command()
@click.argument('file_name', type=str)
def touch(file_name: str):
    """create new file"""

    if 'LICENSE' == file_name.upper():
        # create a license file

        file_utils.create_license()

    else:
        # check if LICENSE_NOTICE file exists
        if file_utils.check_license_notice() is True:
            # create a new file with included license notice

            # check if the programming language of the file is supported
            file_extension: str = file_name.split('.')[-1]
            if lang_utils.is_supported_language(file_extension) is False:
                click.echo(
                    f"ERROR: '{file_extension}' file extension not supported")
                return

            # create new file with license notice included
            file_utils.create_file_with_license_notice(
                file_name, file_extension)

        else:
            # create a regular new file

            file_utils.create_empty_file()



import click
import subprocess
from datetime import datetime
import os

"""
    Copyright 2019 Shabd Saran

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""


def init() -> None:
    """utility function to initialize git repository"""

    try:
        subprocess.call('git init', shell=True)
    except:
        click.echo('ERROR: failed to initialize git repository')
        return


def check_git() -> bool:
    """utility function to check if git repository exists"""

    return os.path.isdir('.git')


def add() -> None:
    """utility function to add all files for staging"""

    subprocess.call('git add .', shell=True)
    subprocess.call('git status', shell=True)


def commit(message: str) -> None:
    """utility function to commit changes"""

    subprocess.call(f"git commit -m '{message}'", shell=True)


def push(branch: str) -> None:
    """utility function to push changes to remote repository"""

    # get all unique remotes associated with current remote repository
    remotes: str = os.popen('git remote -v').read()
    remotes: list = remotes.split('\n')[:-1]
    for i in range(len(remotes)):
        remotes[i] = remotes[i].split('\t')[0]
    remotes = list(set(remotes))

    if 1 == len(remotes):
        # since there is only one remote, push changes
        if branch is None:
            os.system(f'git push {remotes[0]} master')
        else:
            os.system(f'git push {remotes[0]} {branch}')
    else:
        # prompt the user to select one of the remotes
        remote: str = click.prompt(
            'which remote?', default=remote[0], type=click.Choice(remotes, case_sensitive=False))

        # push changes to the given remote
        if branch is None:
            os.system(f'git push {remote} master')
        else:
            os.system(f'git push {remote} {branch}')


def log():
    """utility function to print git repository logs"""

    os.system('git log > git.log && gedit git.log')



import click
from datetime import datetime
import subprocess

# import utils
from utils.language import is_supported_language
import utils.git as git_utils
import utils.file as file_utils


"""
    Copyright 2019 Shabd Saran

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""


@click.group()
def git():
    pass


@git.command()
@click.option('-l', '--language', type=str)
def init(language: str):
    """initialize git repository"""

    # check if programming language is supported
    if language is not None and is_supported_language(language) is not True:
        click.echo(
            f"ERROR: {language} file extension programming language not supported")
        return

    # initialize git repository
    git_utils.init()

    # create LICENSE
    file_utils.create_license()

    # create README.md
    file_utils.create_readme()

    # create .gitignore
    if language is not None:
        file_utils.create_gitignore(language)


@git.command('commit')
@click.option('-m', '--message', required=True, type=str)
@click.option('-b', '--branch', type=str)
def commit(message: str, branch: str):
    """commit changes to git repository"""

    # check if git repository exists
    if git_utils.check_git() is not True:
        click.echo('ERROR: git repository for current directory does not exists')
        return

    # add changes for staging
    git_utils.add()

    # prompt to confirm if the user wants to commit changes
    confirm_commit: bool = click.prompt(
        'confirm commit?', default=True, type=bool)
    if confirm_commit is False:
        return

    # commit changes
    git_utils.commit(message)

    # prompt to check if the user wants to push changes to a remote repository
    confirm_push: bool = click.prompt('push changes?', default=True, type=bool)
    if confirm_push is False:
        return

    # push to remote repository
    git_utils.push(branch)


@git.command()
def log():
    """log git commits"""

    # check if git repository exists
    if git_utils.check_git() is not True:
        click.echo('ERROR: git repository for current directory does not exists')
        return

    # print git repository log
    git_utils.log()
